// Pic.h

// Pic.h

// ##############################################################################################
// Class to simulate two-stream instability in hot, collisionless, unmagnetized, one-dimensional,
// periodic plasma with stationary singly-charged ions via particle-in-cell method
// ##############################################################################################

#ifndef PIC
#define PIC

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <fftw.h>

class Pic
{
  // %%%%%%%%%%
  // Class data
  // %%%%%%%%%%
private:

  // ---------------------
  // Calcuation parameters
  // ---------------------
  int    N;    // Number of electrons
  int    J;    // Number of grid points in x (must be even)
  double L;    // Domain of solution is 0 <= x <= L (where L measured in Debye lengths)
  double vb;   // Beam speed (in units of electron thermal speed)
  double dt;   // Fixed time step (in units of inverse electron plasma frequencies)
  double tmax; // Simulation run from t = 0 to t = tmax (in units of inverse electon plasma frequencies)

  // %%%%%%%%%%%%%%%%%%%%%%
  // Public class functions
  // %%%%%%%%%%%%%%%%%%%%%%
public:

  // Constructor
  Pic ();
  // Destructor
  ~Pic ();
  // Status function    
  void Status ();
  // Solve problem
  void Solve ();
  // Solve previous problem with time running backwards
  void SolveBackwards ();

  // .......................................
  // Functions to set calculation parameters
  // .......................................  
  int SetN    (int);
  int SetJ    (int);
  int SetL    (double);
  int Setvb   (double);
  int Setdt   (double);
  int Settmax (double);

  // %%%%%%%%%%%%%%%%%%%%%%%
  // Private class functions
  // %%%%%%%%%%%%%%%%%%%%%%%
private:

  // Calculate total system energy per electron
  void CalcEnergy (double* Y, double& kinetic, double& total);
  // Write output for nth time step
  void WriteOutput (int step, double* Y);
  // Return random variable distributed according to double-Maxwellian function
  double DoubleMaxwellian ();
  
  // Load electron coordinates and velocities into solution vector
  void LoadSolutionVector (double* r, double* v, double* Y);
  // Unload electron coordinates and velocities from solution vector
  void UnLoadSolutionVector (double* Y, double* r, double* v);
  
  // Evolve electron equations of motion in time from time t to time t + dt
  void Evolve (double& t, double* Y);
  // Calculate electron number density, electric potental, and electric field
  void CalcFields (double* r, double* ne, double* phi, double* E);
  // Calculate right-hand sides of electron equations of motion
  void CalcRhs (double* Y, double* dYdt);

  // Calculate electron number density from electron coordinates
  void Calcne (double* r, double* ne);
  // Calculate electric field from electric potential
  void CalcE (double* phi, double *E);
  
  // Solve Poisson's equation in one dimension via Fourier method
  void Poisson_1D (double* u, double* v);
  // Calculate Fourier transform of real array f in complex array (Fr, Fi)
  void Fourier_Forward (double* f, double* Fr, double* Fi);
  // Calculate inverse Fourier transform of complex array (Fr, Fi) in real array f
  void Fourier_Backward (double* Fr, double* Fi, double* f);
 
  // Open new file for writing
  FILE* OpenFile (char*);
  // Open existing file for reading
  FILE* OpenFiler (char*);
 };

#endif //PIC
